#!/usr/bin/env python3

import json
import logging
import os
import re
import sys


def check_brackets(string):
    opened = 0
    for c in string:
        if c == '(':
            opened += 1
        elif c == ')':
            opened -= 1
    return string + max(0, opened) * ')'


def clean_up_string(string):
    wo_brackets = re.sub(r'\([^\)]*\)', '', string).replace("[", "").replace("]", "").replace(".", "")
    wo_brackets = wo_brackets.replace(' vnd ', ' und ')
    return wo_brackets


def clean_text(e):
    e = re.sub(r'<[^>]*>', r'', e)
    e = check_brackets(e)
    e = e.lstrip('.').lstrip(' ')
    e = re.sub(r'\([^\)]*\)', '', e)
    e = clean_up_string(e)
    e = re.sub(r'\(|\)', '', e)
    e = re.sub(r'\s+', ' ', e)
    return e.strip()


def grep_tag(f, start_tag, end_tag):
    found = re.findall(r'{0}(.*){1}'.format(start_tag, end_tag), f)

    if found:
        if start_tag == '<abstract>':
            found[0] = found[0][3:-4]
    else:
        return dict()

    out_grep = dict()
    for context in found:
        search_context = re.sub(r'\([^\)]*\)', '', context)
        entities = re.findall(r'<persName (authId="[^\"]*")?.*?reg="([^\"]+)"[^>]*>|<placeName (authId="[^\"]*")?.*?reg="([^\"]+)"[^>]*>', search_context)
        types = re.findall(r'<(persName|placeName)', context)
        clean_context = re.sub(r'<[^\>]*>', ' ', context)
        clean_context = clean_up_string(clean_context.strip())
        clean_context = re.sub(r'\s+', ' ', clean_context)
        clean_entities = list()
        ent_ids = list()
        for e in entities:
            current_entity = e[1] if e[1] else e[3]
            current_id = e[0] if e[0] else e[2]
            current_id = current_id.split('=')[-1].strip('"')
            current_entity = clean_text(current_entity)
            clean_entities.append(current_entity)
            ent_ids.append(current_id)

        out_grep[max(out_grep) + 1 if out_grep else 0] = {'context': clean_context, 'entities': clean_entities, 'lang': 'cs', 'types': types, 'ids': ent_ids}
    return out_grep


def extract(filename, out_filename, log_file):
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s | %(levelname)s | %(message)s')
    handler = logging.FileHandler(log_file, 'a', 'UTF-8')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    logger.info('{0}: Entity extraction started'.format(filename))

    f = open(filename, 'r', encoding='UTF-8').read()
    f = re.sub(r'\s+', ' ', f)
    comment = grep_tag(f, '<p n="comment">', '</p>')
    place = grep_tag(f, '<issuePlace>', '</issuePlace>')
    for k,v in place.items():
        comment[k + max(comment) + 1 if comment else k] = v
    
    sigillant = grep_tag(f, '<sigillant>', '</sigillant>')
    for k,v in sigillant.items():
        comment[k + max(comment) + 1 if comment else k] = v
    
    abstract = grep_tag(f, '<abstract>', '</abstract>')
    for k,v in abstract.items():
        comment[k + max(comment) + 1 if comment else k] = v
    
    out_file = open(out_filename, 'w', encoding='UTF-8')
    json.dump(comment, out_file)

    logger.info('{0}: Entity extraction finished'.format(filename))

extract(sys.argv[1], sys.argv[2], sys.argv[3])

